function c = corr_data_uni(dim, disk, pad, nh)
% c = corr_data_uni(dim, disk, pad, nh)  Create a random signal, and impose
% correlation. Then remove multinormality by choosing randomly from 10 neighest
% values in a 5x5 neighbourhood.
% 
% dim controls the size of the field
% disk controls the size of the disc used to filter the field
% pad controls the 
%
% Literature: THE VARIOGRAM AND ITS ESTIMATION, Omre, 1984 
%
% Matt Foster <ee1mpf@bath.ac.uk>

% $LastChangeDate$

% Defaults, these will be overriden if arguments are supplied.

if nargin < 2
    error('Expected at least 2 arguments')
end

if nargin < 3
    pad = 50;
end

if nargin < 4
    nh = 2; % 2*nh+1 square neighbourhood
end

if any(size(disk) ~= 1)
    error('Disc size should be a single integer')
end


% Get some multinormal data
d = corr_data(dim+(2*pad), disk);

% remove multinormality
% Look at local neighbourhood 5x5 mask choose randomly from
% top 10

for ii = pad:size(d,1)-pad-1
    for jj = pad:size(d,nh)-pad-1
        nhood = d(ii-nh:ii+nh, jj-nh:jj+nh);
        sv = sort(nhood(:),'descend');
        pos = ceil(10*rand(1));
        new(ii-pad+1,jj-pad+1) = sv(pos);
    end
end

% Make the histogram look more like omre's
c = abs(reallog(new));
